from channels.consumer import SyncConsumer
from arkindex.images.models import Image, Zone
import logging

ZONE_CHUNK_SIZE = 100

logger = logging.getLogger(__name__)


class ImagesConsumer(SyncConsumer):

    def images_resize(self, message):
        image_id, ratio = (
            message.get('id'),
            message.get('ratio', 1.0),
        )
        if ratio == 1.0:
            # Nothing to do!
            return

        assert ratio > 0, 'Ratio cannot be negative or zero'

        image = Image.objects.get(id=image_id)
        zone_count = image.zones.count()
        if zone_count <= 0:
            return

        logger.info('Resizing {} polygons on {!s} to {} times their original size'.format(zone_count, image.id, ratio))

        # PostgreSQL has an operator to resize bounding boxes, which would be written as
        # queryset.update(box=F('box') * ratio), but it does not work with polygons.
        # We therefore have to do the exact same operation in Python in an asynchronous task.

        for i in range(0, zone_count, ZONE_CHUNK_SIZE):
            zones = image.zones.all()[i:i + ZONE_CHUNK_SIZE].only('id', 'image_id', 'polygon')
            for zone in zones:
                zone.polygon = zone.polygon * ratio
            Zone.objects.bulk_update(zones, ['polygon'])
